from pydantic import UUID4, BaseModel, PositiveInt

from .groups import GroupIn, GroupOut
from .phases import PhaseOut
from .teams import TeamIn, TeamModifyScoreBetIn, TeamWithScoreOut


class ScoreBetIn(BaseModel):
    index: PositiveInt
    team1: TeamIn
    team2: TeamIn
    group: GroupIn


class ScoreBetOut(BaseModel):
    id: UUID4
    locked: bool
    team1: TeamWithScoreOut
    team2: TeamWithScoreOut


class Group(BaseModel):
    id: UUID4


class ScoreBetWithGroupIdOut(BaseModel):
    id: UUID4
    locked: bool
    group: Group
    team1: TeamWithScoreOut
    team2: TeamWithScoreOut


class ScoreBetResponse(BaseModel):
    phase: PhaseOut
    group: GroupOut
    score_bet: ScoreBetOut


class ModifyScoreBetIn(BaseModel):
    team1: TeamModifyScoreBetIn
    team2: TeamModifyScoreBetIn



from typing import List, Optional

from pydantic import UUID4, BaseModel

from .generic import PositiveOrZeroInt


class FlagOut(BaseModel):
    url: str

    class Config:
        orm_mode = True


class TeamIn(BaseModel):
    id: UUID4
    score: Optional[PositiveOrZeroInt]


class TeamOut(BaseModel):
    id: UUID4
    code: str
    description: str
    flag: FlagOut

    class Config:
        orm_mode = True


class AllTeamsResponse(BaseModel):
    teams: List[TeamOut]


class OneTeamResponse(BaseModel):
    team: TeamOut


class TeamWithWonOut(BaseModel):
    id: UUID4
    code: str
    description: str
    flag: FlagOut
    won: Optional[bool]


class TeamWithScoreOut(BaseModel):
    id: UUID4
    code: str
    description: str
    flag: FlagOut
    score: Optional[PositiveOrZeroInt]


class TeamModifyScoreBetIn(BaseModel):
    score: Optional[PositiveOrZeroInt]


class TeamModifyBinaryBetIn(BaseModel):
    id: Optional[UUID4]



from pydantic import BaseModel, PositiveInt

from .generic import PositiveOrZeroInt


class UserResult(BaseModel):
    rank: PositiveInt = 1
    first_name: str
    last_name: str
    full_name: str
    number_match_guess: PositiveOrZeroInt
    number_score_guess: PositiveOrZeroInt
    number_qualified_teams_guess: PositiveOrZeroInt
    number_first_qualified_guess: PositiveOrZeroInt
    number_quarter_final_guess: PositiveOrZeroInt
    number_semi_final_guess: PositiveOrZeroInt
    number_final_guess: PositiveOrZeroInt
    number_winner_guess: PositiveOrZeroInt
    points: float

    class Config:
        orm_mode = True



from typing import List

from pydantic import UUID4, BaseModel

from .phases import PhaseOut


class GroupIn(BaseModel):
    id: UUID4


class GroupOut(BaseModel):
    id: UUID4
    code: str
    description: str

    class Config:
        orm_mode = True


class Phase(BaseModel):
    id: UUID4

    class Config:
        orm_mode = True


class GroupWithPhaseIdOut(BaseModel):
    id: UUID4
    code: str
    phase: Phase
    description: str

    class Config:
        orm_mode = True


class AllGroupsResponse(BaseModel):
    phases: List[PhaseOut]
    groups: List[GroupWithPhaseIdOut]


class GroupResponse(BaseModel):
    phase: PhaseOut
    group: GroupOut


class GroupsByPhaseCodeResponse(BaseModel):
    phase: PhaseOut
    groups: List[GroupOut]



from typing import Optional

from pydantic import UUID4, BaseModel, PositiveInt

from .groups import GroupIn, GroupOut
from .phases import PhaseOut
from .teams import TeamIn, TeamModifyBinaryBetIn, TeamWithWonOut


class BinaryBetIn(BaseModel):
    is_one_won: Optional[bool]
    index: PositiveInt
    team1: TeamIn
    team2: TeamIn
    group: GroupIn


class BinaryBetOut(BaseModel):
    id: UUID4
    locked: bool
    team1: Optional[TeamWithWonOut]
    team2: Optional[TeamWithWonOut]


class Group(BaseModel):
    id: UUID4


class BinaryBetWithGroupIdOut(BaseModel):
    id: UUID4
    locked: bool
    group: Group
    team1: Optional[TeamWithWonOut]
    team2: Optional[TeamWithWonOut]


class BinaryBetResponse(BaseModel):
    phase: PhaseOut
    group: GroupOut
    binary_bet: BinaryBetOut


class ModifyBinaryBetIn(BaseModel):
    is_one_won: Optional[bool]
    team1: Optional[TeamModifyBinaryBetIn]
    team2: Optional[TeamModifyBinaryBetIn]

    class Config:
        extra = "forbid"



from pydantic import BaseModel

from .generic import PositiveOrZeroInt
from .teams import TeamOut


class GroupPositionOut(BaseModel):
    team: TeamOut
    played: PositiveOrZeroInt
    won: PositiveOrZeroInt
    drawn: PositiveOrZeroInt
    lost: PositiveOrZeroInt
    goals_for: PositiveOrZeroInt
    goals_against: PositiveOrZeroInt
    goals_difference: int
    points: PositiveOrZeroInt

    class Config:
        extra = "forbid"
        orm_mode = True



import json
import sys
from configparser import ConfigParser
from datetime import datetime, timezone
from functools import lru_cache
from pathlib import Path
from typing import Callable, Dict

from dateutil import parser
from pydantic import BaseModel, BaseSettings
from sqlalchemy.orm import Session

from yak_server.database.models import UserModel

if sys.version_info >= (3, 9):
    from importlib import resources
else:
    import importlib_resources as resources

from yak_server.helpers.rules import compute_finale_phase_from_group_rank


class RuleContainer(BaseModel):
    config: dict
    function: Callable[[Session, UserModel, dict], None]


RULE_MAPPING = {
    "492345de-8d4a-45b6-8b94-d219f2b0c3e9": compute_finale_phase_from_group_rank,
}


class RuleNotDefined(Exception):
    def __init__(self, rule_id: str) -> None:
        super().__init__(f"Rule not defined: {rule_id}")


class Settings(BaseSettings):
    jwt_secret_key: str
    jwt_expiration_time: int
    competition: str
    lock_datetime: datetime = datetime.now(tz=timezone.utc)
    base_correct_result: int = 0
    multiplying_factor_correct_result: int = 0
    base_correct_score: int = 0
    multiplying_factor_correct_score: int = 0
    team_qualified: int = 0
    first_team_qualified: int = 0
    data_folder: str = ""
    rules: Dict[str, RuleContainer] = {}

    def load_config(self) -> None:
        with resources.as_file(resources.files("yak_server") / "data" / self.competition) as path:
            data_folder = path

        rules = {}

        for rule_file in Path(f"{data_folder}/rules").glob("*.json"):
            rule_id = rule_file.stem

            if rule_id not in RULE_MAPPING:
                raise RuleNotDefined(rule_id)

            with rule_file.open() as rule_content:
                rules[rule_id] = RuleContainer(
                    config=json.loads(rule_content.read()),
                    function=RULE_MAPPING[rule_id],
                )

        config = ConfigParser()
        config.read(f"{data_folder}/config.ini")

        self.lock_datetime = parser.parse(config.get("locking", "datetime"))
        self.base_correct_result = config.getint("points", "base_correct_result")
        self.multiplying_factor_correct_result = config.getint(
            "points",
            "multiplying_factor_correct_result",
        )
        self.base_correct_score = config.getint("points", "base_correct_score")
        self.multiplying_factor_correct_score = config.getint(
            "points",
            "multiplying_factor_correct_score",
        )
        self.team_qualified = config.getint("points", "team_qualified")
        self.first_team_qualified = config.getint("points", "first_team_qualified")
        self.data_folder = data_folder
        self.rules = rules

    class Config:
        env_file = ".env"
        env_file_encoding = "utf-8"


@lru_cache(maxsize=None)
def get_settings() -> Settings:
    settings = Settings()
    settings.load_config()

    return settings
